/*----------------------------------------------------------------------
 * $Id: conversionDlg.cpp,v 1.5 2004/02/13 04:03:37 ganzhorn Exp $      
 *                                                                      
 * Copyright (C)   2000            Salem Ganzhorn <eyekode@yahoo.com>
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#include "conversionDlg.hpp"
#include "local.h"

#include <qhbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qapplication.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qvgroupbox.h>
#include <qgrid.h>
#include <qcheckbox.h>
#include <qlineedit.h>

#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtooltip.h>

#include <iostream>
#include <float.h>
ConversionDlg::ConversionDlg ( QWidget*parent, const char * name ) 
  : QWidget(parent,name,WType_TopLevel)
{
  inValueChanged = false;
  

  QVBoxLayout *vbox = new QVBoxLayout(this);
  QGrid * grid = new QGrid(3,this);
  QHBox * hbox = new QHBox(this);

  grid->setMargin(3);

  QMenuBar *mb= new QMenuBar(this);
  QPopupMenu *optionPopup = new QPopupMenu(mb);
  mb->insertItem( "Options...", optionPopup);

  vbox->addWidget(mb);
  vbox->addWidget(grid);
  vbox->addWidget(hbox);

  new QWidget(grid);
  new QLabel(" Hex:",grid);
  leditors[HexLE] = new QLineEdit(grid);

  new QWidget(grid);
  new QLabel(" ASCII:",grid);
  leditors[AsciiLE] = new QLineEdit(grid);

  byteSwapFlag[IntegerBS] = new QCheckBox("b/s",grid);
  new QLabel(" Int:",grid);
  leditors[IntegerLE] = new QLineEdit(grid);

  byteSwapFlag[FloatBS] = new QCheckBox("b/s",grid);
  new QLabel(" Float:",grid);
  leditors[FloatLE]  = new QLineEdit(grid);

  byteSwapFlag[DoubleBS] = new QCheckBox("b/s",grid);
  new QLabel(" Double:",grid);
  leditors[DoubleLE] = new QLineEdit(grid);

  // make sure to do an update if any of the byte swap flags toggle
  for(int i = MinBS; i < MaxBS; ++i) {
    connect(byteSwapFlag[i],SIGNAL(clicked()),
				   this,SLOT(valueChanged()));
    // add tool tip
    QToolTip::add(byteSwapFlag[i],"Byte Swap data");
  }

  ignoreClear = new QCheckBox("Ignore selection clear", this);
  ignoreClear->setChecked(true);
  optionPopup->insertItem(ignoreClear);
  
  searchForwards = new QCheckBox("Search forwards", this);
  searchForwards->setChecked(true);
  connect(searchForwards,SIGNAL(toggled(bool)),
	  this,SIGNAL(searchForwardsToggled(bool)));
  optionPopup->insertItem(searchForwards);

  QPushButton *pbtn = new QPushButton("Close",hbox);
  connect(pbtn,SIGNAL(clicked()),this,SLOT(close()));
  pbtn = new QPushButton("Search",hbox);
  connect(pbtn,SIGNAL(clicked()),this,SLOT(searchSlot()));

  // setup validators
  leditors[DoubleLE]->setValidator( new QDoubleValidator(this) );
  QDoubleValidator *vd = new QDoubleValidator(this);
  vd->setRange( FLT_MAX, FLT_MAX );
  leditors[FloatLE]->setValidator( vd );
  leditors[IntegerLE]->setValidator( new QIntValidator(this) );
  leditors[HexLE]->setValidator( new HexValidator(this) );

  // setup connections
  connect(leditors[DoubleLE],SIGNAL(textChanged(const QString& )),
	  this,SLOT(doubleChanged(const QString&)) );
  connect(leditors[FloatLE],SIGNAL(textChanged(const QString& )),
	  this,SLOT(floatChanged(const QString&)) );
  connect(leditors[IntegerLE],SIGNAL(textChanged(const QString&)),
  	  this,SLOT(integerChanged(const QString&)) );
  connect(leditors[HexLE],SIGNAL(textChanged(const QString&)),
	  this,SLOT(valueChanged(const QString&)));
  connect(leditors[AsciiLE],SIGNAL(textChanged(const QString&)),
	  this,SLOT(asciiChanged(const QString&)));
  for( int i = MinLE; i < MaxLE; ++i ) {
     connect(leditors[i],SIGNAL(returnPressed()),
	     this,SLOT(searchSlot()));
  }
  
  setCaption("Conversion Dialog");
}

ConversionDlg::~ConversionDlg()
{
  cout << "died" <<endl;
}

void ConversionDlg::searchSlot()
{
  QApplication::setOverrideCursor( Qt::waitCursor );
  emit search( leditors[HexLE]->text() );
  QApplication::restoreOverrideCursor();
}

void ConversionDlg::floatChanged(const QString& str)
{
  char lastCh = str[str.length()-1];
  if( lastCh == 'e' || lastCh == '-' || lastCh == '.' ) {
    return;
  }

  bool   success;
  float value = str.toFloat(&success);

  if( !success && !value ) {
    return;
  }

  // double to hex:
  uchar * ptr = (uchar*) &value;
  vector<uchar> bytes;

  if( byteSwapFlag[FloatBS]->isChecked() ) {
    for( int i = sizeof(float)-1; i > -1; i--)
      bytes.push_back(ptr[i]);
  } else {
    for( unsigned int i = 0; i < sizeof(float); i++ ) 
      bytes.push_back(ptr[i]);
  }

  QString hex;
  Translate::ByteToHex(hex,bytes);

  valueChanged( hex );
}

void ConversionDlg::doubleChanged(const QString& str)
{
  char lastCh = str[str.length()-1];
  if( lastCh == 'e' || lastCh == '-' || lastCh == '.' ) {
    return;
  }

  bool   success;
  double value = str.toDouble(&success);

  if( !success && !value ) {
    return;
  }

  // double to hex:
  uchar * ptr = (uchar*)&value;
  vector<uchar> bytes;

  if(byteSwapFlag[DoubleBS]->isChecked() ) {
    for(int i = sizeof(double)-1; i > -1; --i)
      bytes.push_back(ptr[i]);
  } else {
    for(unsigned int i = 0; i < sizeof(double); i++ )
      bytes.push_back(ptr[i]);
  }
  QString hex;
  Translate::ByteToHex(hex,bytes);

  valueChanged( hex );
}

void ConversionDlg::asciiChanged(const QString &str)
{
  vector<uchar> bytes;
  QString hex;
  Translate::CharToByte(bytes,str);
  Translate::ByteToHex(hex,bytes);

  valueChanged( hex );
}

void ConversionDlg::integerChanged( const QString & str )
{
  bool success;
  int value   = str.toInt(&success);

  uchar * ptr = (uchar*) &value;
  vector<uchar> bytes;

  if( byteSwapFlag[IntegerBS]->isChecked() ) {
    for( int i = sizeof(int)-1; i > -1 ;--i )
      bytes.push_back(ptr[i]);
  } else {
    for( unsigned int i = 0; i < sizeof(int)/sizeof(uchar); i++)
      bytes.push_back(ptr[i]);
  }

  QString hex;
  Translate::ByteToHex(hex,bytes);
  
  valueChanged( hex );
}

// this slot just recalculates conversions
void ConversionDlg::valueChanged()
{
  // make sure the HexLE has focus, so valueChanged(str) updates all fields
  leditors[HexLE]->setFocus();
  valueChanged( leditors[HexLE]->text() );
}
// convert data to hex, then call updateData to set all the other data fields
void ConversionDlg::valueChanged( const QString& str ) 
{
  // don't allow this call to nest
  if(inValueChanged)
    return;

  if( ignoreClear->isChecked() && str.isEmpty()  )
    return;
  
  inValueChanged = true;
  // remmember cursor offset
  int cursor_pos = leditors[HexLE]->cursorPosition();
  // conversion is safe because all chars are less than 'f'
  leditors[HexLE]->setText( str );
  leditors[HexLE]->setCursorPosition( cursor_pos );
  
  vector<uchar> bytes;
  Translate::HexToByte(bytes,str);
  if( !leditors[AsciiLE]->hasFocus() ) {
    
    if(!bytes.size()) {
      for(int i = MinLE; i < MaxLE; ++i)
	leditors[i]->setText( "" );
      inValueChanged = 0;
      return;
    }
    
    // update the ascii entry:
    QString ascii;
    Translate::ByteToChar(ascii,bytes);
    
    leditors[AsciiLE]->setText( ascii );
  }
  
  // scratch conversion vars
  QString strvalue;
  uchar * ucharPtr;
  
  if( !leditors[IntegerLE]->hasFocus() ) {
    // update the int entry:
    // pad right with 0x00
    int intvalue = 0;
    ucharPtr = (uchar*) &intvalue;
    if( byteSwapFlag[IntegerBS]->isChecked() ) {
      for(unsigned int i = 0; (i < sizeof(int)) && (i < bytes.size()); i++)
	ucharPtr[sizeof(int)-1-i] = bytes[i];
    } else {
      memcpy(&intvalue,&bytes.begin()[0],
	     min(sizeof(int),bytes.size()));
    }
    strvalue.setNum(intvalue); 
    leditors[IntegerLE]->setText( strvalue );
  }
  
  if( ! leditors[FloatLE]->hasFocus() ) {
    // update the float entry:
    float fvalue;
    ucharPtr = (uchar*)(&fvalue);
    if( byteSwapFlag[FloatBS]->isChecked() ) {
      for(unsigned int i = 0; i < sizeof(float); ++i) {
	if( i < bytes.size() ) {
	  ucharPtr[sizeof(float)-1-i] = bytes[i];
	} else {
	  ucharPtr[sizeof(float)-1-i] = 0x00;
	}
      }
    } else {
      if(bytes.size() < sizeof(float) ) {
	for(unsigned int i= 0; i < sizeof(float); ++i) {
	  if( i < bytes.size() ) {
	    *ucharPtr++ = bytes[i];
	  } else {
	    *ucharPtr++ = 0x00;
	  }
	}
      } else {
	memcpy(&fvalue,&bytes.begin()[0],sizeof(float));
      }
    }
    strvalue.setNum( fvalue );
    leditors[FloatLE]->setText( strvalue );
  }

  if( ! leditors[DoubleLE]->hasFocus() ) {
    // update the double entry:
    double dvalue;
    ucharPtr = (uchar*)&dvalue;
    if( byteSwapFlag[DoubleBS]->isChecked() ) {
      for(unsigned int i= 0; i < sizeof(double); ++i) {
	if( i < bytes.size() ) {
	  ucharPtr[sizeof(double)-1-i] = bytes[i];
	} else {
	  ucharPtr[sizeof(double)-1-i] = 0x00;
	}
      }
    } else {
      if(bytes.size() < sizeof(double) ) {
	for(unsigned int i= 0; i < sizeof(double); ++i) {
	  if( i < bytes.size() ) {
	    *ucharPtr++ = bytes[i];
	  } else {
	    *ucharPtr++ = 0x00;
	  }
	}
      } else {
	memcpy(&dvalue,&bytes.begin()[0],sizeof(double));
      }
    }
    strvalue.setNum( dvalue );
    leditors[DoubleLE]->setText( strvalue );
  }

  inValueChanged = false;
}


HexValidator::HexValidator( QWidget * parent, const char * name )
  : QValidator(parent,name)
{
}

// allow 0xfff syntax
QValidator::State HexValidator::validate ( QString &str, int &pos ) const
{
  str = str.lower();
  for(int i = 0; i < pos; i++) {
    if( ! ((str[i] >= 'a' && str[i] <= 'f') ||
	   (str[i] >= '0' && str[i] <= '9') ) &&
	! (i == 1 && str[i] == 'x' )
	) {
      return Invalid;
    }
  }
  return Acceptable;
}

