/*----------------------------------------------------------------------
 * $Id: cursor.hpp,v 1.3 2004/02/01 03:06:49 ganzhorn Exp $             
 *                                                                      
 * Copyright (C)   2000            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#ifndef _CURSOR_HPP_
#define _CURSOR_HPP_

#include <sys/types.h>
#include <limits.h>
#include <stdexcept>

#ifdef _LARGEFILE_SOURCE
#define MAX_OFFSET LONG_LONG_MAX
#else
#define MAX_OFFSET LONG_MAX
#endif

class  Cursor {
public:
  Cursor( off_t byteOffset = 0,
	  off_t charOffset = 0,
	  off_t low        = 0,
	  off_t high       = 1,
	  off_t charsPerByte = 2 ); // defaults to hex
  Cursor( const Cursor& c);

  off_t incrByChar( off_t n );
  off_t incrByByte( off_t n );
  off_t decrByChar( off_t n );
  off_t decrByByte( off_t n );

  off_t   byteOffset() const; //returns the current byte offset
  off_t   charOffset() const; //returns the current char offset. (relative)
  off_t   charOffsetAbsolute() const; //returns the current absolute offset.

  bool   setRange( off_t low, off_t high );
  // setCharsPerByte:
  // sideEffect: sets charOffset to min(_charsPerByte-1,_charOffset)
  off_t   setCharsPerByte( off_t charsPerByte );
  void    setOffset( off_t byteOffset, off_t charOffset );

protected:
  // performs: dst = src; and clamps the value to [low,high)
  void assignClamped(off_t& dst,off_t src,off_t low,off_t high);

  enum {
    OutOfRange = -1,
  };
protected:
  off_t _byteOffset;
  off_t _charOffset;
  off_t _low;
  off_t _high;
  off_t _charsPerByte;
};

#endif
