/*----------------------------------------------------------------------
 * $Id: driver.cpp,v 1.6 2003/12/24 16:20:00 ganzhorn Exp $             
 *                                                                      
 * Copyright (C)   2000            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#include <qapplication.h>
#include <stdexcept>
#include <unistd.h>

extern "C" {
  int optind;
};
#include "hexGui.hpp"
#include "local.h"
#include "compareDlg.hpp"

#define STUB(x) extern "C" { void x() {}}
STUB(glXCreateGLXPixmapMESA);
STUB(glXReleaseBuffersMESA);

// this is used for debugging because I cannot seem to figure out how to see
// the asci representation of a QString while in the debugger.
#include <iostream>
void coutAsciText( const QString& str )
{
  cout << &str << " -> \"" << str << "\"" << endl;
}

int main (int argc, char ** argv)
{
  QApplication a(argc,argv);
  // parse the command line and see what major mode we should be in
  int compare = false;
  int c;
  // skip argv[0]
  optind = 1;
  while( EOF != (c = getopt(argc,argv,"c")) ) {
    switch (c) {
    case 'c':
      compare = true;
      break;
    default:
      cerr << "usage: " PROGRAM " [-c [file1 file2]] files..." <<endl;
      exit(1);
      break;
    }
  }
  if( compare ) {
    int nopts = argc - optind;
    // check to see if files were passed in
    if( nopts >= 2 ) {
      (new CompareDlg(argv[optind],argv[optind+1]))->show();
      optind+=2;
    } else {
      (new CompareDlg())->show(); 
    }
    // treat remaining cmdline options as files to be opened.
    while( optind < argc ) {
      HexGui *hg = new HexGui();
      hg->open(argv[optind++]);
      hg->show();
    }
  } else {
  // each remaining argument is assumed to be a file
    if( optind >= argc ) {
      // no files, just open a hexGui
      (new HexGui())->show();
    } else {
      while( optind < argc ) {
	HexGui *hg = new HexGui();
	hg->open(argv[optind++]);
	hg->show();
      }
    }
  }

  a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );
  int ret=0;
  try {
    ret = a.exec();
  } catch (const exception& e) {
    ret = 1;
    cerr << "[error] - unhandled exeption in main:\"" << e.what() << "\"" <<endl;
  }
  return ret;
}
