/*----------------------------------------------------------------------
 * $Id: hexGui.cpp,v 1.11 2004/02/13 03:57:57 ganzhorn Exp $             
 *                                                                      
 * Copyright (C)   2000            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#include "hexGui.hpp"

#include <qhbox.h>
#include <qvbox.h>

#include <qfiledialog.h>
#include <qsizepolicy.h>
#include <qframe.h>
#include <qstatusbar.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qscrollbar.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <qlineedit.h>

#include "expr.h"

extern QApplication * qApp;

HexGui::HexGui (QWidget * parent, 
		const char * name, 
		WFlags f) 
  : QMainWindow(parent,name,f)
{
  QHBox * hbox = new QHBox(this);

  hexEditor = new HexEditor(hbox);
  hexEditor->setSizePolicy( QSizePolicy( QSizePolicy::Expanding,
					 QSizePolicy::Expanding ) );
  conversionDlg = NULL;

  vsb = new QScrollBar(hbox);

  vsb->setRange(0,0);

  setCentralWidget(hbox);

  connect( hexEditor,SIGNAL(rangeChanged(off_t,off_t)),
	   this,SLOT(setScrollBarRange(off_t,off_t)) );
  connect( vsb, SIGNAL(valueChanged(int)),
	   hexEditor,SLOT(setTopLeftToPercent(int)) );
  connect( hexEditor, SIGNAL(topLeftChanged(off_t)),
	   this, SLOT(setScrollBarValue(off_t)) );
  connect( hexEditor, SIGNAL(offsetChanged(off_t)),
	   this, SLOT(setOffsetLabel(off_t)) );

  // setup menu:
  fileMenu = new QPopupMenu(this);
  fileMenu->insertItem("&Open..",this,SLOT(browseLoadFile()));
  fileMenu->insertItem("&Open in new editor...",this,
		       SLOT(browseLoadFileNew()));
  fileMenu->insertSeparator();
  menuIds[SaveMID] = fileMenu->insertItem("&Save",this,SLOT(save()));
  menuIds[SaveAsMID] = fileMenu->insertItem("Save &As...",this,SLOT(saveAs()));
  // only include close functions in menu if this is a toplevel window
  if( isTopLevel() ) {
    fileMenu->insertSeparator();
    if( isTopLevel() )
      fileMenu->insertItem("Close Editor",this,SLOT(close()));
    else
      fileMenu->insertItem("Close File",this,SLOT(closeFile()));
    fileMenu->insertItem("&Exit Application",qApp, SLOT(quit()));
  }
  menuBar()->insertItem("&File", fileMenu );

  // edit pulldown
  editMenu = new QPopupMenu(this);
  menuIds[UndoMID] = editMenu->insertItem("&Undo",hexEditor,SLOT(undo()));
  menuIds[RedoMID] = editMenu->insertItem("&Redo",hexEditor,SLOT(redo()));
  menuBar()->insertItem("&Edit", editMenu );

  // view menu
  viewMenu = new QPopupMenu(this);
  viewMenu->insertItem("&Conversion Dialog...",this,SLOT(showConversionDlg()));
  // menu for selecting editing base (octal/binary/hex)
  QPopupMenu * baseMenu = new QPopupMenu(this);
  baseMenu->insertItem("&Hex",hexEditor,SLOT(setBaseHex()));
  baseMenu->insertItem("&ASCII",hexEditor,SLOT(setBaseASCII()));
  baseMenu->insertItem("&Binary",hexEditor,SLOT(setBaseBinary()));
  baseMenu->insertItem("&Octal",hexEditor,SLOT(setBaseOctal()));
  viewMenu->insertItem("&Editing base", baseMenu);
  // menu for selecting bytes per column
  QPopupMenu * nbytesMenu = new QPopupMenu(this);
  nbytesMenu->insertItem("&1",hexEditor,SLOT(set1BPC()));
  nbytesMenu->insertItem("&2",hexEditor,SLOT(set2BPC()));
  nbytesMenu->insertItem("&4",hexEditor,SLOT(set4BPC()));
  nbytesMenu->insertItem("&8",hexEditor,SLOT(set8BPC()));
  viewMenu->insertItem("&Bytes per column",nbytesMenu);
  menuBar()->insertItem("&View",viewMenu);

  resize(400,196);
  statusBar()->message("Ready",2000);
  statusBar()->addWidget(new QLabel("Cursor Offset:",statusBar()),0,FALSE);
  offsetLineEdit = new QLineEdit(statusBar());
//  offsetLineEdit->setValidator( new HexValidator(offsetLineEdit) );
  connect(offsetLineEdit,SIGNAL(returnPressed()),
	  SLOT(setOffsetFromStatusBar()));
  statusBar()->addWidget(offsetLineEdit,0,FALSE);
  //  progressBar = new QProgressBar(statusBar());
  //  statusBar()->addWidget(progressBar,1,FALSE);
  statusLabel = new QLabel("",statusBar());
  statusBar()->addWidget(statusLabel,1,FALSE);
  statusBar()->setSizeGripEnabled( (bool) f & WType_TopLevel );
  setCaption(PROGRAM_STRING);

  // deactivate save,saveas undo and redo
  fileMenu->setItemEnabled(menuIds[SaveMID],false);
  fileMenu->setItemEnabled(menuIds[SaveAsMID],false);
  editMenu->setItemEnabled(menuIds[UndoMID],false);
  editMenu->setItemEnabled(menuIds[RedoMID],false);
}

HexGui::~HexGui()
{
}

bool HexGui::open(const char * filename) 
{
  if (hexEditor->open(filename)) {
    if( isTopLevel() )
      setCaption(filename);
    else
      statusLabel->setText(filename);
    fileMenu->setItemEnabled(menuIds[SaveMID],true);
    fileMenu->setItemEnabled(menuIds[SaveAsMID],true);
    editMenu->setItemEnabled(menuIds[UndoMID],true);
    editMenu->setItemEnabled(menuIds[RedoMID],true);
    return true;
  } else {
    return false;
  }
}

void HexGui::gotoOffset(off_t offset)
{
  hexEditor->setOffset(offset);
  hexEditor->seeCursor();
  emit offsetChanged(offset);
}

void HexGui::showConversionDlg()
{
  if( !conversionDlg ) {
    conversionDlg = new ConversionDlg(this);
    connect(hexEditor,SIGNAL(selectionChanged(const QString &)),
	    conversionDlg, SLOT(valueChanged(const QString&)) );
    connect(conversionDlg, SIGNAL(search(const QString&)),
	    hexEditor, SLOT(search(const QString&)));
    connect(conversionDlg, SIGNAL(searchForwardsToggled(bool)),
	    hexEditor,SLOT(setSearchForwards(bool)));
  }
  if( conversionDlg->isVisible() ) {
    conversionDlg->raise();
  } else {
    conversionDlg->show();
  }
}

bool HexGui::browseLoadFile()
{
  QString filename = QFileDialog::getOpenFileName();
  if( filename.isNull() )
    return false;
  return open(filename.latin1());
}
bool HexGui::browseLoadFileNew()
{
  QString filename = QFileDialog::getOpenFileName();
  if( filename.isNull() )
    return false;
  HexGui * hg = new HexGui();
  bool retval = hg->open( filename.latin1() );
  hg->show();
  return retval;
}

bool HexGui::save()
{
  statusBar()->message("saveing " + hexEditor->filename()) ;
  bool retval = hexEditor->save();
  statusBar()->message("done.",2000);
  return retval;
}

bool HexGui::saveAs()
{
  QString filename = QFileDialog::getSaveFileName();
  if( filename.isNull() ) 
    return false;
  statusBar()->message("saveing to \"" + filename + "\"...");
  bool retval = hexEditor->save(filename);
  statusBar()->message("done.",2000);
  setCaption(filename);
  return retval;
}

int HexGui::closeFile()
{
  return closeFile( false );
}
int HexGui::closeFile( bool force )
{
  return hexEditor->closeFile(force);
}

void HexGui::closeEvent( QCloseEvent * ce )
{
  if( hexEditor->closeFile() == QMessageBox::Cancel ) 
    ce->ignore();
  else
    ce->accept();
}

void HexGui::setScrollBarRange(off_t low, off_t high)
{
   (void)low;(void)high;
   // range must be contained in the space of an integer, just do 100
   // increments
   vsb->setRange(0,100);
}
void HexGui::setScrollBarValue(off_t pos)
{
  // pos is the topLeft pos, set the scrollbar to the
  // location of the last byte on the page
  // Note: offsetToPercent now rounds up, so we don't
  // have to worry about if this is the topLeft or bottom right
  vsb->setValue(hexEditor->offsetToPercent(pos));
}
void HexGui::setOffsetLabel(off_t pos)
{
  QString label;
  char    buffer[64];
#if _LARGEFILE_SOURCE
  sprintf(buffer,"0x%llx",pos);
#else
  sprintf(buffer,"0x%lx",pos);
#endif
  label = buffer;
  offsetLineEdit->setText(label);
}

const QString& HexGui::filename(void) const
{
  static QString value = "";
  if(!hexEditor)
    return (value="");
  return (value=hexEditor->filename());
}

void HexGui::setOffsetFromStatusBar()
{
  // remove leading "0x" if necessary
  off_t o;
  if( !expr_eval( offsetLineEdit->text().latin1(),o ) || o < 0 ) {
     QApplication::beep();
  } else {
     gotoOffset(o);
  }
  return;
  bool  ok;
  off_t offset;
  QString value = offsetLineEdit->text();
  if( value.length() > 2 && value[1] == 'x' ) {
     value = value.mid(2);
     offset = value.toInt(&ok,16);
  } else {
     offset = value.toInt(&ok,10);
     if( !ok ) {
	offset = value.toInt(&ok,16);
     }
  }
  if( !ok ) {
     cerr << "Error converting offset: " << offsetLineEdit->text() << endl;
  } else {
     gotoOffset(offset);
  }
}

off_t HexGui::offset() const
{
  return hexEditor->offset();
}

Reader * HexGui::reader()
{
  return hexEditor->reader();
}

void HexGui::setSelection(off_t start,off_t stop)
{
  hexEditor->setSelection(HexEditor::SelectionStart,start);
  hexEditor->setSelection(HexEditor::SelectionEnd,stop);
}
