/*----------------------------------------------------------------------
 * $Id: hexGui.hpp,v 1.3 2003/12/24 16:20:00 ganzhorn Exp $             
 *                                                                      
 * Copyright (C)   2000            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#ifndef _GUI_HPP_
#define _GUI_HPP_

#include <qmainwindow.h>

#include "hexEditor.hpp"
#include "conversionDlg.hpp"
#include "local.h"

class QScrollBar;
class QPopupMenu;
class QString;
class QLabel;
//class QProgressBar;

class HexGui : public QMainWindow {
  Q_OBJECT
public:
  HexGui( QWidget * parent = 0,  
	  const char * name = 0,
	  WFlags f = WType_TopLevel );
  ~HexGui();

  const QString& filename(void) const;
  // see through call to isModified in hexEditor 
  bool isModified() const {return hexEditor->isModified();}
  // public access functions
  off_t offset() const;
  Reader * reader();

signals:
  void offsetChanged(off_t offset);

public slots:
// sets selection to [start,stop)
  void setSelection(off_t start, off_t stop);
  bool browseLoadFile();
  bool browseLoadFileNew();
  bool open(const char * filename);
  void showConversionDlg();
  void gotoOffset(off_t offset);
  void setOffsetFromStatusBar(void);

  bool save();
  bool saveAs();
  void setScrollBarRange(off_t low, off_t high);
  void setScrollBarValue(off_t pos);
  void setOffsetLabel(off_t pos);
  void closeEvent(QCloseEvent *ce);
  // had to trivially overload closeFile() == closeFile(false) for Qt slots
  int  closeFile();
  int  closeFile(bool force);

protected:
  enum MenuID_e {
    UndoMID,
    RedoMID,
    SaveMID,
    SaveAsMID,
    TotalMID,
  };

  QLabel        *statusLabel;
  QLineEdit     *offsetLineEdit;
  //  QProgressBar  *progressBar;
  HexEditor     *hexEditor;
  ConversionDlg *conversionDlg;
  QScrollBar    *vsb;
  int            menuIds[TotalMID];
  QPopupMenu    *fileMenu;
  QPopupMenu    *editMenu;
  QPopupMenu    *viewMenu;
};

#endif
