/*----------------------------------------------------------------------
 * $Id: offsetConstraint.cpp,v 1.5 2003/12/24 16:20:00 ganzhorn Exp $
 *                                                                      
 * Copyright (C)   2000            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#include <qhbox.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qsizepolicy.h>

#include <algorithm> // for min<>()

#include "local.h"
#include "offsetConstraint.hpp"


OffsetConstraint::OffsetConstraint( QWidget * parent,
				    const char * name )
  : QVGroupBox(parent,name)
{
  QHBox * hbox = new QHBox(this);
  new QLabel("X =",hbox);
  stride = new QSpinBox(1,16,1,hbox);
  connect(stride,SIGNAL(valueChanged(int)),
	  this,SLOT(setStride(int)));

  hbox = new QHBox(this);
  new QLabel("Y =",hbox);
  offset = new QSpinBox(0,16,1,hbox);

  stride->setValue(1);
  setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed));
}

// public interface

int OffsetConstraint::getStride(void) const
{
  return stride->text().toInt();
}

int OffsetConstraint::getOffset(void) const
{
  return offset->text().toInt();
}

off_t OffsetConstraint::next(off_t off) const
{
  return( ((off-getOffset())/getStride()+1)*getStride()+getOffset() );
}

off_t OffsetConstraint::prev(off_t off) const
{
  return( ((off-getOffset())/getStride() - 1)*getStride()+getOffset() );
}

// public slots
void OffsetConstraint::setOffset(int x)
{
  offset->setValue( min(x,getStride()-1) );
}

void OffsetConstraint::setStride(int x)
{
  // constrain x to (0,MAXINT)
  if( x < 1 ) {
    stride->setValue(1);
    return;
  }

  // constrain offset to [0,x)

  // setRange() does not seem to work as the documentation says, so
  // we have to manually change the value
  if( offset->text().toInt() >= x )
    offset->setValue(x-1);
  offset->setRange(0,x-1);
  return;
}
  
