/*----------------------------------------------------------------------
 * $Id: save.cpp,v 1.3 2003/12/24 16:20:00 ganzhorn Exp $               
 *                                                                      
 * Copyright (C)   2000            Salem Ganzhorn <eyekode@yahoo.com> 
 *                                                                      
 * This file may be distributed and/or modified under the terms of the  
 * Q Public License as defined by Trolltech AS of Norway and appearing  
 * in the file LICENSE.QPL included in the packaging of this file.      
 *                                                                      
 *----------------------------------------------------------------------*/

#include <errno.h>
#include <string>
#include <vector>
#include <qmessagebox.h>
#include <sys/types.h>

#include "translate.hpp"
#include "local.h"
#include "delta.hpp"
#include "reader.hpp"

extern int errno;

bool writeDeltas( const string& filename , DeltaMap& dmap )
{
  FILE* fptr = fopen( filename.c_str(), "r+" );
  if( NULL == fptr ) {
    QString message = "Error opening \"";
    message += filename.c_str();
    message += "\": ";
    message += strerror(errno);
    QMessageBox::critical( NULL, PROGRAM_STRING,
			   message,
			   "Ok");
    return false;
  }

  const Delta * deltaPtr;
  for(off_t offset = dmap.lower_bound(0);
      offset != -1;
      offset = dmap.lower_bound(offset+1)) {
    if( fseek(fptr,offset,SEEK_SET) ) {
      fclose(fptr);
      QString message = "Error seeking to offset ";
      message += offset;
      message += ": ";
      message += strerror(errno);
      QMessageBox::critical( NULL, PROGRAM_STRING,
			     message,
			     "Ok");
      return false;
    }
    deltaPtr = dmap.search(offset);
    fwrite( &(deltaPtr->newData()[0]), sizeof(uchar) , 
	    deltaPtr->newData().size(), fptr);
  }
  fclose(fptr);
  return true;
}
